"""Gcenter API class to configure the Gcenter WEBui."""
from __future__ import annotations
from .decorators import minimum_version, check_auth
from .errors import AuthenticationError, ApiError
from .utils import is_int
from datetime import datetime
import json
import logging
import urllib3
import requests

logger = logging.getLogger("GWAPI")


class GcenterRequests():
    """Allows to configure the Gcenter WEBui through HTTP interaction.

    Attributes:
        ip (str): IP address of the Gcenter.

    """

    PROXIES = {
        "http": "",
        "https": ""
    }

    def __init__(self, ip: str) -> None:
        """Initialize session to the Gcenter API.

        Args:
            ip (str): IP address of the Gcenter.
            user (User): User to be used to connect to the API.

        """
        urllib3.disable_warnings()
        self.ip = ip

    def _gen_request_kwargs(self,
                            endpoint: str,
                            data: dict,
                            is_json: bool,
                            is_params: bool,
                            headers: dict,
                            cookies: dict,
                            redirects: bool) -> dict:
        if is_json and is_params:
            raise TypeError("Json data cannot be used with params string.")
        kwargs = {
            "url": "https://{}{}".format(self.ip, endpoint),
            "headers": headers,
            "cookies": cookies,
            "verify": False,
            "proxies": self.PROXIES,
            "allow_redirects": redirects
        }
        if is_json:
            kwargs["json"] = data
        elif is_params:
            kwargs["params"] = data
        else:
            kwargs["data"] = data
        return kwargs

    def _get(self, endpoint: str,
             data: dict = {},
             is_json: bool = True,
             is_params: bool = False,
             headers: dict = None,
             cookies: dict = None,
             redirects: bool = True) -> tuple:
        """Wrapper of the get requests.

        Args:
            endpoint: URL endpoint in format /XX/YY/ZZ.
            data: Dictionnary format data.
            redirects: Set to True if redirection is allowed and False instead.

        Returns:
            Return a tuple with format::

            (
                "response in string format",
                status code,
                "reason for that status code"
            )

        """
        kwargs = self._gen_request_kwargs(
            endpoint=endpoint,
            data=data,
            is_json=is_json,
            is_params=is_params,
            headers=headers,
            cookies=cookies,
            redirects=redirects
        )
        response = requests.get(**kwargs)
        return (response.headers, response.text, response.status_code,
                response.reason)

    def _post(self, endpoint: str,
              data: dict,
              is_json: bool = True,
              is_params: bool = False,
              headers: dict = None,
              cookies: dict = None,
              redirects: bool = True) -> tuple:
        """Wrapper of the post requests.

        Args:
            endpoint: URL endpoint in format /XX/YY/ZZ.
            data: Dictionnary format data.
            redirects: Set to True if redirection is allowed and False instead.

        Returns:
            Return a tuple with format::

            (
                "response in string format",
                status code,
                "reason for that status code"
            )

        """
        kwargs = self._gen_request_kwargs(
            endpoint=endpoint,
            data=data,
            is_json=is_json,
            is_params=is_params,
            headers=headers,
            cookies=cookies,
            redirects=redirects
        )
        response = requests.post(**kwargs)
        return (response.headers, response.text, response.status_code,
                response.reason)

    def _put(self, endpoint: str,
             data: dict,
             is_json: bool = True,
             is_params: bool = False,
             headers: dict = None,
             cookies: dict = None,
             redirects: bool = True) -> tuple:
        """Wrapper of the post requests.

        Args:
            endpoint: URL endpoint in format /XX/YY/ZZ.
            data: Dictionnary format data.
            redirects: Set to True if redirection is allowed and False instead.

        Returns:
            Return a tuple with format::

            (
                "response in string format",
                status code,
                "reason for that status code"
            )

        """
        kwargs = self._gen_request_kwargs(
            endpoint=endpoint,
            data=data,
            is_json=is_json,
            is_params=is_params,
            headers=headers,
            cookies=cookies,
            redirects=redirects
        )
        response = requests.put(**kwargs)
        return (response.headers, response.text, response.status_code,
                response.reason)

    def _delete(self, endpoint: str,
                headers: dict = None,
                cookies: dict = None,
                redirects: bool = True) -> tuple:
        """Wrapper of the delete requests.

        Args:
            endpoint: URL endpoint in format /XX/YY/ZZ.
            redirects: Set to True if redirection is allowed and False instead.

        Returns:
            Return a tuple with format::

            (
                "response in string format",
                status code,
                "reason for that status code"
            )

        """
        kwargs = self._gen_request_kwargs(
            endpoint=endpoint,
            data={},
            is_json=False,
            is_params=False,
            headers=headers,
            cookies=cookies,
            redirects=redirects
        )
        response = requests.delete(**kwargs)
        return (response.headers, response.text, response.status_code,
                response.reason)


class GcenterApi(GcenterRequests):
    """Allows to configure the Gcenter WEBui through the API.

    Attributes:
        ip (str): Gcenter IP address.
        token (str): Gcenter API token.
        token_expiration_date (str): Timestamp in string ISO 8601 format.
        headers (str): Headers to be used with the Gcenter API.
        version (str): Gcenter version in format: 2.5.3.X.

    """

    PROXIES = {
        "http": "",
        "https": ""
    }

    def __init__(self, ip: str, version: str) -> None:
        """Initialize the GcenterApi class.

        Args:
            ip: IP address of the Gcenter.
            version: Gcenter version in format: 2.5.3.X.

        """
        super().__init__(ip=ip)
        self.token = None
        self.token_expiration_date = None
        self.headers = {}
        self.version = version

    def auth(self, user: str, password: str) -> bool:
        """Authentication through the Gcenter API.

        Args:
            user: Gcenter WEBui username.
            password: Gcenter WEBui password.

        Returns:
            Return true if authenticated.

        Raises:
            ApiError: If status_code != 200.

        """
        request_data = {
            "username": user,
            "password": password
        }
        response = self._post(
            endpoint="/api/auth/login",
            data=request_data
        )
        if response[2] == 200:
            logger.debug("Authentication on gcenter {} with user {} [OK]".
                         format(self.ip, user))
            self.token = json.loads(response[1])["token"]
            self.token_expiration_date = json.loads(response[1])["expiration_date"]
            self.headers["API-KEY"] = self.token
            return True
        else:
            raise AuthenticationError(
                "Authentication on gcenter {} with user {} [FAILED]".
                format(self.ip, user), *response[1:]
            )

    @check_auth
    @minimum_version("2.5.3.101")
    def get_serial_number(self) -> str:
        """Get Gcenter serial number.

        Returns:
            Return the serial number in XXXXXXX format.

        Raises:
            ApiError: If status_code != 200.

        """
        response = self._get(endpoint="/api/status/gcenter", headers=self.headers)
        if response[2] == 200:
            logger.debug("Get serial number on gcenter {} [OK]".format(self.ip))
            return json.loads(response[1])["serial_number"]
        else:
            raise ApiError(
                "Get serial number on gcenter {} [FAILED]".
                format(self.ip), *response[1:]
            )

    @check_auth
    @minimum_version("2.5.3.101")
    def get_licence(self) -> dict:
        """Get Gcenter licence details.

        Returns:
            Return licence informations in format::

            {
              "key": "",
              "license_expiry_alert": X,
              "details": {
                "end_date": "",
                "days_left": X,
                "registered_owner": "",
                "machine_learning": "",
                "antimalware_engines": X,
                "serial_number": "",
                "sigflow": "",
                "registered_mail": "",
                "valid": "",
                "expired": "",
                "retroact": "",
                "nozomi": "",
                "malcore": "",
                "max_gcaps": X,
                "model": "",
                "codebreaker": "",
                "license_expiry_alert": X,
                "cie": "",
                "start_date": "",
                "full": ""
              }
            }

        Raises:
            ApiError: If status_code != 200.

        """
        response = self._get(endpoint="/api/license/", headers=self.headers)
        if response[2] == 200:
            logger.debug("Get licence info on gcenter {} [OK]".format(self.ip))
            return json.loads(response[1])
        else:
            raise ApiError(
                "Get licence info on gcenter {} [FAILED]".
                format(self.ip), *response[1:]
            )

    @check_auth
    @minimum_version("2.5.3.101")
    def set_licence(self, key: str, expiry_alert: int = 90) -> bool:
        """Set Gcenter licence key and expiry alert.

        Args:
            key: Licence key in string format.
            expiry_alert: The number of days before having an expiry alert between 1 and 90.
                          For example, if set to 7, the user will start receiving
                          alerts 7 days before expiration.

        Returns:
            Return True if licence is valid and False instead.

        Raises:
            ApiError: If status_code != 200.

        """
        if expiry_alert not in range(1, 91):
            raise ValueError("Licence expiry alert must be between 1 and 90.")
        request_data = {
            "key": key,
            "license_expiry_alert": expiry_alert
        }
        response = self._put(endpoint="/api/license/",
                             data=request_data, headers=self.headers)
        if response[2] == 200:
            if json.loads(response[1])["details"]["valid"] is True:
                logger.debug("Set licence on gcenter {} [OK]".format(self.ip))
            else:
                logger.debug("Set licence on gcenter {} [FAILED]".format(self.ip))
            return json.loads(response[1])["details"]["valid"]
        else:
            raise ApiError(
                "Set licence on gcenter {} [FAILED]".
                format(self.ip), *response[1:]
            )

    @check_auth
    @minimum_version("2.5.3.101")
    def get_gcaps(self) -> list:
        """Get all the Gcaps linked to the Gcenter.

        Returns:
            Return a list of gcaps with format::

            [
                {
                  "id": 0,
                  "fqdn": "string",
                  "is_paired": true,
                  "last_rule_update": "string",
                  "status": "online"
                }
            ]

        Raises:
            ApiError: If status_code != 200.

        """
        response = self._get(endpoint="/api/gcaps", headers=self.headers)
        if response[2] == 200:
            logger.debug("Get Gcaps for gcenter {} [OK]".format(self.ip))
            return json.loads(response[1])["results"]
        else:
            raise ApiError(
                "Get Gcaps for gcenter {} [FAILED]".
                format(self.ip), *response[1:]
            )

    @check_auth
    @minimum_version("2.5.3.101")
    def get_gcap_by_id(self, gcap_id: int) -> dict:
        """Get a Gcap pairing configuration by id.

        Args:
            gcap_id: Gcap identifier. Can be found with the get_gcaps() function.

        Returns:
            Return a gcap pairing configuration with format::

            {
              "id": 0,
              "fqdn": "string",
              "is_paired": true,
              "last_rule_update": "string",
              "status": "online"
            }

        Raises:
            ApiError: If status_code != 200.

        """
        response = self._get(endpoint="/api/gcaps/{}".format(gcap_id), headers=self.headers)
        if response[2] == 200:
            logger.debug("Get Gcap {} for gcenter {} [OK]".format(gcap_id, self.ip))
            return json.loads(response[1])
        else:
            raise ApiError(
                "Get Gcap {} for gcenter {} [FAILED]".
                format(gcap_id, self.ip), *response[1:]
            )

    @check_auth
    @minimum_version("2.5.3.101")
    def get_gcap_ruleset(self, gcap_id: int) -> dict:
        """Get a full Gcap profile configuration.

        Args:
            gcap_id: Gcap identifier. Can be found with the get_gcaps() function.

        Returns:
            Return a Gcap profile configuration with format::

            {
              "variables": "string",
              "flow_timeouts": [
                {
                  "protocol": "string",
                  "new": 0,
                  "established": 0,
                  "closed": 0,
                  "emergency_new": 0,
                  "emergency_established": 0,
                  "emergency_closed": 0
                }
              ],
              "bpf_filter": {
                "id": 0,
                "default_vlan": 0,
                "gcap_rules_mgmt": 0
              },
              "applied": true,
              "timestamp_modif": "2021-07-02T09:48:26.323Z",
              "multitenant": true,
              "is_by_interface": true,
              "max_pending_packets": 0,
              "file_store_stream_depth_enable": true,
              "file_store_stream_depth_mb": 0,
              "stream_memcap_b": 0,
              "stream_prealloc_sessions": 0,
              "stream_reassembly_memcap_b": 0,
              "stream_reassembly_depth_mb": 0,
              "stream_reassembly_toserver_chunk_size_b": 0,
              "stream_reassembly_toclient_chunk_size_b": 0,
              "flow_memcap": 0,
              "flow_prealloc": 0,
              "stream_reassembly_randomize_chunk_size": true,
              "xff_enable": true,
              "xff_mode": "extra-data",
              "xff_deployment": "reverse",
              "xff_header": "string",
              "payload": true,
              "payload_buffer_size": 0,
              "payload_printable": true,
              "packet": true,
              "file_resend_interval": 0,
              "http_body": true,
              "http_body_printable": true,
              "ftp_memcap": 0,
              "smb_stream_depth": 0,
              "http_enable": false,
              "dns_udp_enable": false,
              "dns_tcp_enable": false,
              "dns_enable": false,
              "tls_enable": false,
              "smtp_enable": false,
              "smb_enable": false,
              "ssh_enable": false,
              "netflow_enable": false,
              "dnp3_enable": false,
              "ftp_enable": false,
              "dhcp_enable": false,
              "ikev2_enable": false,
              "krb5_enable": false,
              "nfs_enable": false,
              "tftp_enable": false,
              "parsing_dcerpc_enabled": 0,
              "parsing_dnp3_enabled": 0,
              "parsing_dns_udp_enabled": 0,
              "parsing_dns_tcp_enabled": 0,
              "parsing_ftp_enabled": 0,
              "parsing_http_enabled": 0,
              "parsing_modbus_enabled": 0,
              "parsing_smb_enabled": 0,
              "parsing_smtp_enabled": 0,
              "parsing_ssh_enabled": 0,
              "parsing_tls_enabled": 0,
              "parsing_dhcp_enabled": 0,
              "parsing_ikev2_enabled": 0,
              "parsing_krb5_enabled": 0,
              "parsing_nfs_enabled": 0,
              "parsing_ntp_enabled": 0,
              "parsing_tftp_enabled": 0,
              "files_hash": "md5",
              "files_rules": "string"
            }

        Raises:
            ApiError: If status_code != 200.

        """
        response = self._get(endpoint="/api/gcaps/{}/ruleset".format(gcap_id),
                             headers=self.headers)
        if response[2] == 200:
            logger.debug("Get Gcaps rulesets for Gcenter {} [OK]".format(self.ip))
            return json.loads(response[1])
        else:
            raise ApiError(
                "Get Gcaps rulesets for Gcenter {} [FAILED]".
                format(self.ip), *response[1:]
            )

    @check_auth
    @minimum_version("2.5.3.101")
    def get_gcap_interfaces(self, gcap_id: int) -> list:
        """Get the Gcap's available network interfaces.

        Args:
            gcap_id: Gcap identifier. Can be found with the get_gcaps() function.

        Returns:
            Return all the Gcap available network interfaces with format::

            [
                {
                  "enabled": false,
                  "name": "string",
                  "mtu": 0,
                  "is_cluster": false,
                  "cluster_interfaces": [
                    "string"
                  ]
                }
            ]

        Raises:
            ApiError: If status_code != 200.

        """
        response = self._get(endpoint="/api/gcaps/{}/interfaces".format(gcap_id),
                             headers=self.headers)
        if response[2] == 200:
            logger.debug("Get Gcaps interfaces for Gcenter {} [OK]".format(self.ip))
            return json.loads(response[1])["results"]
        else:
            raise ApiError(
                "Get Gcaps interfaces for Gcenter {} [FAILED]".
                format(self.ip), *response[1:]
            )

    @check_auth
    @minimum_version("2.5.3.101")
    def get_gcap_template(self) -> str:
        """Get the Gcaps template.

        Returns:
            Return the active gcap template. Value can be:
            [minimal, default, lpm, paranoid, intuitio]

        Raises:
            ApiError: If status_code != 200.

        """
        response = self._get(endpoint="/api/gcaps/profile-template/", headers=self.headers)
        if response[2] == 200:
            logger.debug("Get Gcaps template on gcenter {} [OK]".format(self.ip))
            return json.loads(response[1])["profile"]
        else:
            raise ApiError(
                "Get Gcaps template on gcenter {} [FAILED]".
                format(self.ip), *response[1:]
            )

    @check_auth
    @minimum_version("2.5.3.101")
    def set_gcap_template(self, template: str = "default") -> dict:
        """Set the Gcaps template.

        Args:
            template: Profile between [minimal, default, lpm, paranoid, intuitio]

        Returns:
            Return the active gcap template. Value can be:
            [minimal, default, lpm, paranoid, intuitio]

        Raises:
            ApiError: If status_code != 200.

        """
        request_data = {
            "profile": template
        }
        response = self._put(endpoint="/api/gcaps/profile-template/",
                             data=request_data, headers=self.headers)
        if response[2] == 200:
            logger.debug("Set Gcaps template to {} on gcenter {} [OK]".
                         format(template, self.ip))
            return json.loads(response[1])["profile"]
        else:
            raise ApiError(
                "Set Gcaps template to {} on gcenter {} [FAILED]".
                format(template, self.ip), *response[1:]
            )

    @check_auth
    @minimum_version("2.5.3.101")
    def get_gcap_profile(self, gcap_id: int) -> dict:
        """Get a Gcap base variables configuration.

        Args:
            gcap_id: Gcap identifier. Can be found with the get_gcaps() function.

        Returns:
            Return a Gcap base variables configuration with format::

            {
              "files_hash": [
                "md5"
              ],
              "max_pending_packets": 0,
              "file_store_stream_depth_enable": true,
              "file_store_stream_depth_mb": 0,
              "stream_memcap_b": 0,
              "stream_prealloc_sessions": 0,
              "stream_reassembly_memcap_b": 0,
              "stream_reassembly_depth_mb": 0,
              "stream_reassembly_toserver_chunk_size_b": 0,
              "stream_reassembly_toclient_chunk_size_b": 0,
              "flow_memcap": 0,
              "flow_prealloc": 0,
              "stream_reassembly_randomize_chunk_size": true,
              "xff_enable": true,
              "xff_mode": "extra-data",
              "xff_deployment": "reverse",
              "xff_header": "string",
              "payload": true,
              "payload_buffer_size": 0,
              "payload_printable": true,
              "packet": true,
              "file_resend_interval": 0,
              "http_body": true,
              "http_body_printable": true,
              "ftp_memcap": 0,
              "smb_stream_depth": 0,
              "http_enable": false,
              "dns_udp_enable": false,
              "dns_tcp_enable": false,
              "tls_enable": false,
              "smtp_enable": false,
              "smb_enable": false,
              "ssh_enable": false,
              "netflow_enable": false,
              "dnp3_enable": false,
              "ftp_enable": false,
              "dhcp_enable": false,
              "ikev2_enable": false,
              "krb5_enable": false,
              "nfs_enable": false,
              "tftp_enable": false,
              "parsing_dcerpc_enabled": 0,
              "parsing_dnp3_enabled": 0,
              "parsing_dns_udp_enabled": 0,
              "parsing_dns_tcp_enabled": 0,
              "parsing_ftp_enabled": 0,
              "parsing_http_enabled": 0,
              "parsing_modbus_enabled": 0,
              "parsing_smb_enabled": 0,
              "parsing_smtp_enabled": 0,
              "parsing_ssh_enabled": 0,
              "parsing_tls_enabled": 0,
              "parsing_dhcp_enabled": 0,
              "parsing_ikev2_enabled": 0,
              "parsing_krb5_enabled": 0,
              "parsing_nfs_enabled": 0,
              "parsing_ntp_enabled": 0,
              "parsing_tftp_enabled": 0
            }

        Raises:
            ApiError: If status_code != 200.

        """
        response = self._get(endpoint="/api/gcaps/{}/profile/".format(gcap_id),
                             headers=self.headers)
        if response[2] == 200:
            logger.debug("Get profile for gcap {} on gcenter {} [OK]".
                         format(gcap_id, self.ip))
            return json.loads(response[1])
        else:
            raise ApiError(
                "Get profile for gcap {} on gcenter {} [FAILED]".
                format(gcap_id, self.ip), *response[1:]
            )

    @check_auth
    @minimum_version("2.5.3.101")
    def set_gcap_profile(self, gcap_id: int,
                         proto: str = None,
                         parsing: bool = None,
                         logging: bool = None,
                         **profile_variables) -> dict:
        """Set a Gcap parsing/logging configuration for a specified protocol.

        Args:
            gcap_id: Gcap identifier. Can be found with the get_gcaps() function.
            proto: Gcap protocol found in the parsing tab of the base variables Gcap configuration.
            parsing: Set to True to enable parsing for a protocol, False to disable it
                     and None to not change the actual configuration.
                     Valids parsing protocols are: "dcerpc", "dhcp", "dnp3", "dns_udp",
                     "dns_tcp", "ftp", "http", "ikev2", "krb5", "modbus", "nfs", "ntp", "smb",
                     "smtp", "ssh", "tftp", "tls".
            logging: Set to True to enable logging for a protocol, False to disable it
                     and None to not change the actual configuration.
                     Valids parsing protocols are: "dhcp", "dnp3", "dns_udp", "dns_tcp",
                     "ftp", "http", "ikev2", "krb5", "netflow", "nfs", "smb", "smtp",
                     "ssh", "tftp", "tls".

        Returns:
            Return a Gcap base variables configuration with format::

            {
              "files_hash": [
                "md5"
              ],
              "max_pending_packets": 0,
              "file_store_stream_depth_enable": true,
              "file_store_stream_depth_mb": 0,
              "stream_memcap_b": 0,
              "stream_prealloc_sessions": 0,
              "stream_reassembly_memcap_b": 0,
              "stream_reassembly_depth_mb": 0,
              "stream_reassembly_toserver_chunk_size_b": 0,
              "stream_reassembly_toclient_chunk_size_b": 0,
              "flow_memcap": 0,
              "flow_prealloc": 0,
              "stream_reassembly_randomize_chunk_size": true,
              "xff_enable": true,
              "xff_mode": "extra-data",
              "xff_deployment": "reverse",
              "xff_header": "string",
              "payload": true,
              "payload_buffer_size": 0,
              "payload_printable": true,
              "packet": true,
              "file_resend_interval": 0,
              "http_body": true,
              "http_body_printable": true,
              "ftp_memcap": 0,
              "smb_stream_depth": 0,
              "http_enable": false,
              "dns_udp_enable": false,
              "dns_tcp_enable": false,
              "tls_enable": false,
              "smtp_enable": false,
              "smb_enable": false,
              "ssh_enable": false,
              "netflow_enable": false,
              "dnp3_enable": false,
              "ftp_enable": false,
              "dhcp_enable": false,
              "ikev2_enable": false,
              "krb5_enable": false,
              "nfs_enable": false,
              "tftp_enable": false,
              "parsing_dcerpc_enabled": 0,
              "parsing_dnp3_enabled": 0,
              "parsing_dns_udp_enabled": 0,
              "parsing_dns_tcp_enabled": 0,
              "parsing_ftp_enabled": 0,
              "parsing_http_enabled": 0,
              "parsing_modbus_enabled": 0,
              "parsing_smb_enabled": 0,
              "parsing_smtp_enabled": 0,
              "parsing_ssh_enabled": 0,
              "parsing_tls_enabled": 0,
              "parsing_dhcp_enabled": 0,
              "parsing_ikev2_enabled": 0,
              "parsing_krb5_enabled": 0,
              "parsing_nfs_enabled": 0,
              "parsing_ntp_enabled": 0,
              "parsing_tftp_enabled": 0
            }

        Raises:
            ApiError: If status_code != 200.

        """
        request_data = self.get_gcap_profile(gcap_id)
        if proto is not None:
            if parsing is not None:
                request_data["parsing_{}_enabled".format(proto.lower())] = int(parsing)
            if logging is not None:
                request_data["{}_enable".format(proto.lower())] = logging
        for key, value in profile_variables.items():
            if key not in request_data:
                logger.warning(
                    "Profile variable {} not found for gcap {} on gcenter {} [WARNING]".
                    format(key, gcap_id, self.ip)
                )
            else:
                request_data[key] = value
        response = self._put(endpoint="/api/gcaps/{}/profile/".format(gcap_id),
                             data=request_data, headers=self.headers)
        if response[2] == 200:
            if proto is not None:
                logger.debug(
                    "Set proto {} parsing {} logging {} for gcap {} on gcenter {} [OK]".
                    format(proto, parsing, logging, gcap_id, self.ip)
                )
            for key, value in profile_variables.items():
                logger.debug(
                    "Set profile variable {} with value {} for gcap {} on gcenter {} [OK]".
                    format(key, value, gcap_id, self.ip)
                )
            return json.loads(response[1])
        else:
            raise ApiError(
                "Set profile variables for gcap {} on gcenter {} [FAILED]".
                format(gcap_id, self.ip), *response[1:]
            )

    @check_auth
    @minimum_version("2.5.3.101")
    def get_gcap_single_tenant(self, gcap_id: int) -> dict:
        """Get the Gcap single-tenant configuration.

        Args:
            gcap_id: Gcap identifier. Can be found with the get_gcaps() function.

        Returns:
            Return the Gcap single-tenant configuration with format::

            {
                "enabled": false,
                "ruleset": 0,
                "enable_shellcode": true,
                "enable_powershell": true
            }

        Raises:
            ApiError: If status_code != 200.

        """
        response = self._get(endpoint="/api/gcaps/{}/single-tenant/".
                             format(gcap_id), headers=self.headers)
        if response[2] == 200:
            logger.debug("Get profile for gcap {} on gcenter {} [OK]".
                         format(gcap_id, self.ip))
            return json.loads(response[1])
        else:
            raise ApiError(
                "Get profile for gcap {} on gcenter {} [FAILED]".
                format(gcap_id, self.ip), *response[1:]
            )

    @check_auth
    @minimum_version("2.5.3.101")
    def set_gcap_single_tenant(self, gcap_id: int,
                               enabled: bool = True,
                               ruleset_id: int = 1,
                               shellcode: bool = True,
                               powershell: bool = True) -> dict:
        """Set the Gcap single-tenant configuration.

        Args:
            gcap_id: Gcap identifier. Can be found with the get_gcaps() function.
            enabled: Set to True to enable single tenant and False instead.
            ruleset_id: Ruleset ID to be apply to the Gcap.
            shellcode: Set to True to enable shellcode and False instead.
            powershell: Set to True to enable powershell and False instead.

        Returns:
            Return the Gcap single-tenant configuration with format::

            {
                "enabled": false,
                "ruleset": 0,
                "enable_shellcode": true,
                "enable_powershell": true
            }

        Raises:
            ApiError: If status_code != 200.

        """
        request_data = {
            "enabled": enabled,
            "ruleset": ruleset_id,
            "enable_shellcode": shellcode,
            "enable_powershell": powershell
        }
        response = self._put(endpoint="/api/gcaps/{}/single-tenant/".
                             format(gcap_id), data=request_data, headers=self.headers)
        if response[2] == 200:
            logger.debug("Set single tenant for gcap {} on gcenter {} [OK]".
                         format(gcap_id, self.ip))
            return json.loads(response[1])
        else:
            raise ApiError(
                "Set single tenant for gcap {} on gcenter {} [FAILED]".
                format(gcap_id, self.ip), *response[1:]
            )

    @check_auth
    @minimum_version("2.5.3.101")
    def get_gcap_multi_tenant(self, gcap_id: int) -> dict:
        """Set the Gcap multi-tenant configuration.

        Args:
            gcap_id: Gcap identifier. Can be found with the get_gcaps() function.

        Returns:
            Return the Gcap multi-tenant configuration with format::

            {
              "enabled": false,
              "ruleset": [
                {
                  "id": 0,
                  "ruleset": 0,
                  "codebreaker_shellcode": true,
                  "codebreaker_powershell": true,
                  "name": "string"
                }
              ],
              "is_by_interface": false
            }

        Raises:
            ApiError: If status_code != 200.

        """
        response = self._get(endpoint="/api/gcaps/{}/multi-tenant/".
                             format(gcap_id), headers=self.headers)
        if response[2] == 200:
            logger.debug("Get profile for gcap {} on gcenter {} [OK]".
                         format(gcap_id, self.ip))
            return json.loads(response[1])
        else:
            raise ApiError(
                "Get profile for gcap {} on gcenter {} [FAILED]".
                format(gcap_id, self.ip), *response[1:]
            )

    @check_auth
    @minimum_version("2.5.3.101")
    def disable_gcap_multi_tenant(self, gcap_id: int) -> dict:
        """Disable the Gcap multi-tenant configuration.

        Args:
            gcap_id: Gcap identifier. Can be found with the get_gcaps() function.

        Returns:
            Return the Gcap multi-tenant configuration with format::

            {
              "enabled": false,
              "ruleset": [],
              "is_by_interface": false
            }

        Raises:
            ApiError: If status_code != 200.

        """
        request_data = {
            "enabled": False,
            "ruleset": [],
            "is_by_interface": True
        }
        response = self._put(endpoint="/api/gcaps/{}/multi-tenant/".
                             format(gcap_id), data=request_data, headers=self.headers)
        if response[2] == 200:
            logger.debug("Disable multi-tenant on gcap {} on gcenter {} [OK]".
                         format(gcap_id, self.ip))
            return json.loads(response[1])
        else:
            raise ApiError(
                "Disable multi-tenant on gcap {} on gcenter {} [FAILED]".
                format(gcap_id, self.ip), *response[1:]
            )

    @check_auth
    @minimum_version("2.5.3.101")
    def reset_gcap_tenant(self, gcap_id: int) -> bool:
        """Reset single/multi tenant configuration.

        Args:
            gcap_id: Gcap identifier. Can be found with the get_gcaps() function.

        Returns:
            Return True if the configuration reset was successful and False instead.

        Raises:
            ApiError: If status_code != 200.

        """
        response = self._post(endpoint="/api/gcaps/{}/reset_tenant".
                              format(gcap_id), data="", headers=self.headers)
        if response[2] == 200:
            logger.debug("Reset single/multi tenant on gcap {} on gcenter {} [OK]".
                         format(gcap_id, self.ip))
            return True if "done" in json.loads(response[1])["details"] else False
        else:
            raise ApiError(
                "Reset single/multi tenant on gcap {} on gcenter {} [FAILED]".
                format(gcap_id, self.ip), *response[1:]
            )

    @check_auth
    @minimum_version("2.5.3.101")
    def set_gcap_multi_tenant_interface(self, gcap_id: int,
                                        interface: str,
                                        ruleset_id: int = 1,
                                        shellcode: bool = True,
                                        powershell: bool = True) -> dict:
        """Set the Gcap multi-tenant interface configuration.

        Args:
            gcap_id: Gcap identifier. Can be found with the get_gcaps() function.
            interface: Interface name. Ex: mon0, mon1.
            ruleset_id: Ruleset ID to be apply to the Gcap.
            shellcode: Set to True to enable shellcode and False instead.
            powershell: Set to True to enable powershell and False instead.

        Returns:
            Return the Gcap multi-tenant configuration with format::

            {
              "enabled": false,
              "ruleset": [
                {
                  "id": 0,
                  "ruleset": 0,
                  "codebreaker_shellcode": true,
                  "codebreaker_powershell": true,
                  "name": "string"
                }
              ],
              "is_by_interface": false
            }

        Raises:
            ApiError: If status_code != 200.

        """
        ruleset_previous = self.get_gcap_multi_tenant(gcap_id)
        if "ruleset" in ruleset_previous.keys():
            ruleset_list = [prev for prev in ruleset_previous["ruleset"]
                            if not is_int(prev["name"])
                            and prev["name"] != interface
                            and "default" != prev["name"]]
            ruleset_list = [{key: value for key, value in ruleset.items()
                             if key != "id"} for ruleset in ruleset_list]
        else:
            ruleset_list = []
        ruleset_list.append({
            "ruleset": ruleset_id,
            "codebreaker_shellcode": shellcode,
            "codebreaker_powershell": powershell,
            "name": interface
        })
        request_data = {
            "enabled": True,
            "ruleset": ruleset_list,
            "is_by_interface": True
        }
        response = self._put(endpoint="/api/gcaps/{}/multi-tenant/".
                             format(gcap_id), data=request_data, headers=self.headers)
        if response[2] == 200:
            logger.debug("Set multi-tenant for interface {} on gcap {} [OK]".
                         format(interface, gcap_id))
            return json.loads(response[1])
        else:
            raise ApiError(
                "Set multi-tenant for interface {} on gcap {} [FAILED]".
                format(interface, gcap_id), *response[1:]
            )

    @check_auth
    @minimum_version("2.5.3.101")
    def set_gcap_multi_tenant_vlan(self, gcap_id: int,
                                   vlan: str,
                                   ruleset_id: int = 1,
                                   shellcode: bool = True,
                                   powershell: bool = True) -> None:
        """Set the Gcap multi-tenant vlan configuration.

        Args:
            gcap_id: Gcap identifier. Can be found with the get_gcaps() function.
            vlan: Vlan name from 1 to 4096. Ex: default, 110, 120.
            ruleset_id: Ruleset ID to be apply to the Gcap.
            shellcode: Set to True to enable shellcode and False instead.
            powershell: Set to True to enable powershell and False instead.

        Returns:
            Return the Gcap multi-tenant configuration with format::

            {
              "enabled": false,
              "ruleset": [
                {
                  "id": 0,
                  "ruleset": 0,
                  "codebreaker_shellcode": true,
                  "codebreaker_powershell": true,
                  "name": "string"
                }
              ],
              "is_by_interface": false
            }

        Raises:
            ApiError: If status_code != 200.

        """
        ruleset_previous = self.get_gcap_vlans(gcap_id)
        ruleset_list = [prev for prev in ruleset_previous
                        if prev["name"] != str(vlan)]
        ruleset_list = [{key: value for key, value in ruleset.items()
                         if key != "id"} for ruleset in ruleset_list]
        if len(ruleset_list) == 0:
            raise ValueError(
                "You must create at least two vlan to configure multi-tenant by vlan !"
            )
        ruleset_list.append({
            "ruleset": ruleset_id,
            "codebreaker_shellcode": shellcode,
            "codebreaker_powershell": powershell,
            "name": vlan
        })
        request_data = {
            "enabled": True,
            "ruleset": ruleset_list,
            "is_by_interface": False
        }
        response = self._put(endpoint="/api/gcaps/{}/multi-tenant/".
                             format(gcap_id), data=request_data, headers=self.headers)
        if response[2] == 200:
            logger.debug("Set multi-tenant for vlan {} on gcap {} [OK]".
                         format(vlan, gcap_id))
            return json.loads(response[1])
        else:
            raise ApiError(
                "Set multi-tenant for vlan {} on gcap {} [FAILED]".
                format(vlan, gcap_id), *response[1:]
            )

    @check_auth
    @minimum_version("2.5.3.101")
    def get_gcap_vlans(self, gcap_id: int) -> list:
        """Get the Gcap vlans configuration.

        Args:
            gcap_id: Gcap identifier. Can be found with the get_gcaps() function.

        Returns:
            Return the Gcap vlans configuration with format::

            [
                {
                  "id": 0,
                  "ruleset": 0,
                  "codebreaker_shellcode": true,
                  "codebreaker_powershell": true,
                  "name": "string"
                }
            ]

        Raises:
            ApiError: If status_code != 200.

        """
        response = self._get(endpoint="/api/gcaps/{}/vlans".format(gcap_id),
                             headers=self.headers)
        if response[2] == 200:
            logger.debug("Get vlans config on gcap {} [OK]".format(gcap_id))
            return json.loads(response[1])["results"]
        else:
            raise ApiError(
                "Get vlans config on gcap {} [FAILED]".
                format(gcap_id), *response[1:]
            )

    @check_auth
    @minimum_version("2.5.3.101")
    def set_gcap_vlan(self, gcap_id: int,
                      vlan: str,
                      ruleset_id: int = 1,
                      shellcode: bool = True,
                      powershell: bool = True) -> None:
        """Create a vlan configuration. Cannot update a vlan configuration.

        Args:
            gcap_id: Gcap identifier. Can be found with the get_gcaps() function.
            vlan: Vlan name from 1 to 4096. Ex: default, 110, 120.
            ruleset_id: Ruleset ID to be apply to the Gcap.
            shellcode: Set to True to enable shellcode and False instead.
            powershell: Set to True to enable powershell and False instead.

        Returns:
            Return the Gcap vlans configuration with format::

            {
              "id": 0,
              "ruleset": 0,
              "codebreaker_shellcode": true,
              "codebreaker_powershell": true,
              "name": "string"
            }

        Raises:
            ApiError: If status_code != 200.

        """
        for vl in self.get_gcap_vlans(gcap_id=gcap_id):
            if vl["name"] == vlan:
                raise ValueError("Vlan {} already exists for gcap {} [FAILED]".
                                 format(vlan, gcap_id))
        request_data = {
            "ruleset": ruleset_id,
            "codebreaker_shellcode": shellcode,
            "codebreaker_powershell": powershell,
            "name": vlan
        }
        response = self._post(endpoint="/api/gcaps/{}/vlans".format(gcap_id),
                              data=request_data, headers=self.headers)
        if response[2] == 201:
            logger.debug("Create vlan {} for gcap {} [OK]".format(vlan, gcap_id))
            return json.loads(response[1])
        else:
            raise ApiError(
                "Create vlan {} for gcap {} [FAILED]".
                format(vlan, gcap_id), *response[1:]
            )

    @check_auth
    @minimum_version("2.5.3.101")
    def delete_gcap_vlan(self, gcap_id: int, vlan: str) -> bool:
        """Delete a vlan configuration.

        Args:
            gcap_id: Gcap identifier. Can be found with the get_gcaps() function.
            vlan: Vlan name from 1 to 4096. Ex: 110, 120.

        Returns:
            Return True if successful and False instead.

        Raises:
            ApiError: If status_code != 200.

        """
        for vl in self.get_gcap_vlans(gcap_id=gcap_id):
            if vl["name"] == vlan:
                vlan_id = vl["id"]
        if vlan_id is None:
            raise ValueError("Vlan {} not found for gcap {} [FAILED]".
                             format(vlan, gcap_id))
        response = self._delete(endpoint="/api/gcaps/{}/vlans/{}".format(gcap_id, vlan_id),
                                headers=self.headers)
        if response[2] == 204:
            logger.debug("Delete vlan {} for gcap {} [OK]".format(vlan, gcap_id))
            return True
        else:
            raise ApiError(
                "Delete vlan {} for gcap {} [FAILED]".
                format(vlan, gcap_id), *response[1:]
            )

    @check_auth
    @minimum_version("2.5.3.101")
    def apply_gcap(self, gcap_id: int) -> bool:
        """Save the Gcap profiles configuration.

        Args:
            gcap_id: Gcap identifier. Can be found with the get_gcaps() function.

        Returns:
            Return True if successful and False instead.

        Raises:
            ApiError: If status_code != 200.

        """
        response = self._post(endpoint="/api/gcaps/{}/apply".format(gcap_id),
                              data="", headers=self.headers)
        if response[2] == 200:
            logger.debug("Apply configuration for gcap {} [OK]".format(gcap_id))
            return True if "success" in json.loads(response[1])["detail"] else False
        else:
            raise ApiError(
                "Apply configuration for gcap {} [FAILED]".
                format(gcap_id), *response[1:]
            )

    @minimum_version("2.5.3.101")
    def get_api_status(self) -> bool:
        """Get the Gcenter API status. No authentication is needed.

        Returns:
            Return True if status is OK and False instead.

        Raises:
            ApiError: If status_code != 200.

        """
        response = self._get(endpoint="/api/status/", headers=self.headers)
        if response[2] == 200:
            logger.debug("Get status for Gcenter {} [OK]".format(self.ip))
            return json.loads(response[1])["success"]
        else:
            raise ApiError(
                "Get status for Gcenter {} [FAILED]".
                format(self.ip), *response[1:]
            )

    @check_auth
    @minimum_version("2.5.3.101")
    def get_gcenter_status(self) -> dict:
        """Get the Gcenter version and serial number.

        Returns:
            Return a dictionnary with format::

            {
              "version": "2.5.3.101-XXXX",
              "serial_number": "XXXXXX"
            }

        Raises:
            ApiError: If status_code != 200.

        """
        response = self._get(endpoint="/api/status/gcenter", headers=self.headers)
        if response[2] == 200:
            logger.debug("Get status for Gcenter {} [OK]".format(self.ip))
            return json.loads(response[1])
        else:
            raise ApiError(
                "Get status for Gcenter {} [FAILED]".
                format(self.ip), *response[1:]
            )

    @check_auth
    @minimum_version("2.5.3.101")
    def get_healthchecks_status(self) -> dict:
        """Get the Gcenter healthchecks status.

        Returns:
            Return a dictionnary with format::

            {
              "healthy": [Good,Warning,Bad],
              "errors": []
            }

        Raises:
            ApiError: If status_code != 200.

        """
        response = self._get(endpoint="/api/status/healthchecks", headers=self.headers)
        if response[2] == 200:
            logger.debug("Get healthchecks status for Gcenter {} [OK]".format(self.ip))
            return json.loads(response[1])
        else:
            raise ApiError(
                "Get healthchecks status for Gcenter {} [FAILED]".
                format(self.ip), *response[1:]
            )

    @check_auth
    @minimum_version("2.5.3.101")
    def get_updates_status(self) -> dict:
        """Get the Gcenter updates status.

        Returns:
            Return a dictionnary with format::

            {
              "healthy": [Good,Warning,Bad],
              "errors": []
            }

        Raises:
            ApiError: If status_code != 200.

        """
        response = self._get(endpoint="/api/status/updates", headers=self.headers)
        if response[2] == 200:
            logger.debug("Get updates status for Gcenter {} [OK]".format(self.ip))
            return json.loads(response[1])
        else:
            raise ApiError(
                "Get updates status for Gcenter {} [FAILED]".
                format(self.ip), *response[1:]
            )

    @minimum_version("2.5.3.101")
    def get_user_status(self) -> dict:
        """Get the Gcenter user status. No authentication is needed.

        Returns:
            Return a dictionnary with format::

            {
              "message": "string",
              "authenticated": false
            }

        Raises:
            ApiError: If status_code != 200.

        """
        response = self._get(endpoint="/api/status/user", headers=self.headers)
        if response[2] == 200:
            logger.debug("Get user status for Gcenter {} [OK]".format(self.ip))
            return json.loads(response[1])
        else:
            raise ApiError(
                "Get user status for Gcenter {} [FAILED]".
                format(self.ip), *response[1:]
            )

    @check_auth
    @minimum_version("2.5.3.101")
    def get_users(self) -> list:
        """Get the Gcenter users data.

        Returns:
            Return a list of users in dictionnary with format::

            [
                {
                  "id": X,
                  "username": "XXX",
                  "roles": [
                    {
                      "name": "XXX"
                    }
                  ],
                  "groups": [
                    {
                      "id": X,
                      "name": "XXX"
                    }
                  ]
                }
            ]

        Raises:
            ApiError: If status_code != 200.

        """
        response = self._get(endpoint="/api/users", headers=self.headers)
        if response[2] == 200:
            logger.debug("Get users for Gcenter {} [OK]".format(self.ip))
            return json.loads(response[1])["results"]
        else:
            raise ApiError(
                "Get users for Gcenter {} [FAILED]".
                format(self.ip), *response[1:]
            )

    @check_auth
    @minimum_version("2.5.3.101")
    def get_user_by_id(self, user_id: int) -> dict:
        """Get the Gcenter user data by id.

        Args:
            gcap_id: Gcap identifier. Can be found with the get_gcaps() function.

        Returns:
            Return a user dictionnary with format::

            {
              "id": X,
              "username": "XXX",
              "roles": [
                {
                  "name": "XXX"
                }
              ],
              "groups": [
                {
                  "id": X,
                  "name": "XXX"
                }
              ]
            }

        Raises:
            ApiError: If status_code != 200.

        """
        response = self._get(endpoint="/api/users/{}".format(user_id), headers=self.headers)
        if response[2] == 200:
            logger.debug("Get user {} for Gcenter {} [OK]".format(user_id, self.ip))
            return json.loads(response[1])
        else:
            raise ApiError(
                "Get user {} for Gcenter {} [FAILED]".
                format(user_id, self.ip), *response[1:]
            )

    @check_auth
    @minimum_version("2.5.3.101")
    def get_sigflow_rulesets(self, with_files: bool = False) -> list:
        """Get the Gcenter Sigflow rulesets configuration.

        Args:
            with_files: Set to True to get only the ruleset with generated files
                        and False instead.

        Returns:
            Return a list of ruleset in dictionnary with format::

            [
                {
                  "id": X,
                  "name": "XXX",
                  "descr": "XXX",
                  "created_date": "XXX",
                  "has_files": bool
                }
            ]

        Raises:
            ApiError: If status_code != 200.

        """
        request_data = {"with_files": with_files}
        response = self._get(endpoint="/api/sigflow/rulesets",
                             data=request_data, is_json=False,
                             is_params=True, headers=self.headers)
        if response[2] == 200:
            logger.debug("Get sigflow rulesets for Gcenter {} [OK]".format(self.ip))
            return json.loads(response[1])["results"]
        else:
            raise ApiError(
                "Get sigflow rulesets for Gcenter {} [FAILED]".
                format(self.ip), *response[1:]
            )

    @check_auth
    @minimum_version("2.5.3.101")
    def get_gcenter_alerts(self, page: int = 1,
                           page_size: int = 100,
                           date_from: str = None,
                           date_to: str = None,
                           gcap_id: str = "all",
                           ip: str = None,
                           src_ip: str = None,
                           dest_ip: str = None,
                           sort_by: str = "date_desc",
                           risk_min: int = 0,
                           risk_max: int = 10,
                           name: str = None,
                           description: str = None) -> list:
        """Get the last alerts sent by Gcenter's Gcaps.

        Args:
            page: A page number within the paginated result set.
            page_size: Number of results to return per page.
            date_from: Date must be a valid ISO-8601 string.
            date_to: Date must be a valid ISO-8601 string.
            gcap_id: Gcap ID. Set to "all" to get all gcaps alerts.
            ip: if set, get all the alerts related to this ip.
            src_ip: If set, get all the alerts where this ip is the source of this alert.
            dest_ip: If set, get all the alerts where this ip is the target of this alert.
            sort_by: date_asc, date_desc, risk_asc, risk_desc.
            risk_min: If set, get all the alerts with greater risk than this value.
            risk_max: If set, get all the alerts with lower risk than this value.
            name: Filter the alerts by name.
            description: Filter the alerts by its description.

        Returns:
            Return the last alerts sent by Gcenter's Gcaps with format::

            [
                {
                  "id": "string",
                  "name": "string",
                  "date": "2021-07-02T10:40:31.233Z",
                  "gcap": {
                    "id": 0,
                    "fqdn": "string",
                    "is_paired": true,
                    "last_rule_update": "string",
                    "status": "online"
                  },
                  "description": "string",
                  "src_ip": "string",
                  "dest_ip": "string",
                  "risk": 0,
                  "type": "sigflow"
                }
            ]

        Raises:
            ApiError: If status_code != 200.

        """
        request_data = {k: v for k, v in locals().items() if k != "self"}
        alerts_sort = ["date_asc", "date_desc", "risk_asc", "risk_desc"]
        try:
            if date_from is not None:
                datetime.fromisoformat(date_from)
            if date_to is not None:
                datetime.fromisoformat(date_to)
        except Exception:
            raise ValueError("Date must be a valid ISO-8601 string.")
        if sort_by not in alerts_sort:
            raise ValueError("Alerts sorting must be between: {}.".format(alerts_sort))
        response = self._get(endpoint="/api/alerts",
                             data=request_data, is_json=False,
                             is_params=True, headers=self.headers)
        if response[2] == 200:
            logger.debug("Get alerts for Gcenter {} [OK]".format(self.ip))
            return json.loads(response[1])["results"]
        else:
            raise ApiError(
                "Get alerts for Gcenter {} [FAILED]".
                format(self.ip), *response[1:]
            )

    @check_auth
    @minimum_version("2.5.3.101")
    def get_gcenter_clusters_alerts(self, page: int = 1,
                                    page_size: int = 100,
                                    date_from: str = None,
                                    date_to: str = None,
                                    gcap_id: str = "all",
                                    ip: str = None,
                                    src_ip: str = None,
                                    dest_ip: str = None,
                                    sort_by: str = "all",
                                    frequency: str = "day") -> list:
        """Get clusters alerts. A Cluster is defined by a number of alerts that
        occurred in the same laps of time (the last hour or the last day for example),
        and which are all related to the same ip address.

        Args:
            page: A page number within the paginated result set.
            page_size: Number of results to return per page.
            date_from: Date must be a valid ISO-8601 string.
            date_to: Date must be a valid ISO-8601 string.
            gcap_id: Gcap ID. Set to "all" to get all gcaps alerts.
            ip: if set, get all the alerts related to this ip.
            src_ip: If set, get all the alerts where this ip is the source of this alert.
            dest_ip: If set, get all the alerts where this ip is the target of this alert.
            sort_by: all, dest, src.
            frequency: See the cluster alerts by "day" or "hour".

        Returns:
            Return clusters alerts with format::

            [
                {
                  "id": "string",
                  "ip": "string",
                  "number_alerts": 0,
                  "average_risk": 0,
                  "risk_score": 0,
                  "date": "2021-07-02T10:38:13.173Z",
                  "description": "string",
                  "malcore_codebreaker": 0,
                  "sigflow": 0,
                  "type_ip": "src",
                  "gcap": {
                    "id": 0,
                    "fqdn": "string",
                    "is_paired": true,
                    "last_rule_update": "string",
                    "status": "online"
                  }
                }
            ]

        Raises:
            ApiError: If status_code != 200.

        """
        request_data = {k: v for k, v in locals().items() if k != "self"}
        alerts_sort = ["all", "dest", "src"]
        alerts_frequency = ["day", "hour"]
        try:
            if date_from is not None:
                datetime.fromisoformat(date_from)
            if date_to is not None:
                datetime.fromisoformat(date_to)
        except Exception:
            raise ValueError("Date must be a valid ISO-8601 string: 2000-01-01T12:52:29.59")
        if sort_by not in alerts_sort:
            raise ValueError("Alerts sorting must be between: {}.".format(alerts_sort))
        if frequency not in alerts_frequency:
            raise ValueError("Alerts frequency must be between: {}.".format(alerts_frequency))
        response = self._get(endpoint="/api/alerts/clusters",
                             data=request_data, is_json=False,
                             is_params=True, headers=self.headers)
        if response[2] == 200:
            logger.debug("Get clusters alerts for Gcenter {} [OK]".format(self.ip))
            return json.loads(response[1])["results"]
        else:
            raise ApiError(
                "Get clusters alerts for Gcenter {} [FAILED]".
                format(self.ip), *response[1:]
            )

    @check_auth
    @minimum_version("2.5.3.101")
    def get_sigflow_ruleset_by_id(self, ruleset_id: int) -> dict:
        """Get a specified ruleset configuration.

        Args:
            ruleset_id: Ruleset ID.

        Returns:
            Return a ruleset configuration with format::

            {
              "id": X,
              "name": "XXX",
              "descr": "XXX",
              "created_date": "XXX",
              "has_files": bool
            }

        Raises:
            ApiError: If status_code != 200.

        """
        response = self._get(endpoint="/api/sigflow/rulesets/{}".
                             format(ruleset_id), headers=self.headers)
        if response[2] == 200:
            logger.debug("Get ruleset {} for Gcenter {} [OK]".format(ruleset_id, self.ip))
            return json.loads(response[1])
        else:
            raise ApiError(
                "Get ruleset {} for Gcenter {} [FAILED]".
                format(ruleset_id, self.ip), *response[1:]
            )

    @check_auth
    @minimum_version("2.5.3.101")
    def get_malcore_settings(self) -> dict:
        """Get malcore settings.

        Returns:
            Return malcore settings with format::

            {
              "retroact_number_of_days_between_rescans": X,
              "retroact_number_of_rescan": X,
              "gbox_analysis": bool
            }

        Raises:
            ApiError: If status_code != 200.

        """
        response = self._get(endpoint="/api/malcore/settings/global", headers=self.headers)
        if response[2] == 200:
            logger.debug("Get malcore settings on gcenter {} [OK]".format(self.ip))
            return json.loads(response[1])
        else:
            raise ApiError(
                "Get malcore settings on gcenter {} [FAILED]".
                format(self.ip), *response[1:]
            )

    @check_auth
    @minimum_version("2.5.3.101")
    def set_malcore_settings(self, days: int, rescan: int, gbox: bool = False) -> dict:
        """Set malcore settings.

        Args:
            days: Number of days between rescans.
            rescan: Number of rescan.
            gbox: Set to True to enabled gbox analysis and False instead.

        Returns:
            Return malcore settings with format::

            {
              "retroact_number_of_days_between_rescans": X,
              "retroact_number_of_rescan": X,
              "gbox_analysis": bool
            }

        Raises:
            ApiError: If status_code != 200.

        """
        request_data = {
            "retroact_number_of_days_between_rescans": days,
            "retroact_number_of_rescan": rescan,
            "gbox_analysis": gbox
        }
        response = self._put(endpoint="/api/malcore/settings/global",
                             data=request_data, headers=self.headers)
        if response[2] == 200:
            logger.debug("Set malcore settings on gcenter {} [OK]".format(self.ip))
            return json.loads(response[1])
        else:
            raise ApiError(
                "Set malcore settings on gcenter {} [FAILED]".
                format(self.ip), *response[1:]
            )

    @check_auth
    @minimum_version("2.5.3.101")
    def get_gcenter_interfaces(self) -> list:
        """Get Gcenter network interfaces settings.

        Returns:
            Return a list of dictionnary with format::

            [
                {
                  "name": "XXX",
                  "fullname": "XXX - Y.Y.Y.Y",
                  "hostname": "Y.Y.Y.Y"
                }
            ]

        Raises:
            ApiError: If status_code != 200.

        """
        response = self._get(endpoint="/api/net/interfaces", headers=self.headers)
        if response[2] == 200:
            logger.debug("Get network interfaces on gcenter {} [OK]".
                         format(self.ip))
            return json.loads(response[1])["results"]
        else:
            raise ApiError(
                "Get network interfaces on gcenter {} [FAILED]".
                format(self.ip), *response[1:]
            )

    @check_auth
    @minimum_version("2.5.3.101")
    def get_gcenter_interface_by_name(self, name: str) -> dict:
        """Get Gcenter network interface settings by name.

        Args:
            name: Gcenter network interface name. Ex: mgmt0.

        Returns:
            Return a dictionnary with format::

            {
              "name": "XXX",
              "fullname": "XXX - Y.Y.Y.Y",
              "hostname": "Y.Y.Y.Y"
            }

        Raises:
            ApiError: If status_code != 200.

        """
        response = self._get(endpoint="/api/net/interfaces/{}".format(name),
                             headers=self.headers)
        if response[2] == 200:
            logger.debug("Get network interfaces {} on gcenter {} [OK]".
                         format(name, self.ip))
            return json.loads(response[1])
        else:
            raise ApiError(
                "Get network interfaces {} on gcenter {} [FAILED]".
                format(name, self.ip), *response[1:]
            )

    @check_auth
    @minimum_version("2.5.3.101")
    def get_certificate_data(self, certificate: str, key: str) -> dict:
        """Get the certificate data.

        Args:
            certificate: Certificate in PEM string format.
            key: Private key in PEM string format.

        Returns:
            Return a dictionnary with format::

            {
                "version": X,
                "version_hexa": X,
                "serial_num": "XX:XX:XX:XX:XX:XX:XX:XX",
                "sign_algo": "XXX",
                "issuer": "C=X ST=X L=X O=X OU=X CN=X",
                "not_before": "Oct 02 15:03:33 2020 GMT",
                "not_after": "Jun 11 15:03:33 2034 GMT",
                "subject": "C=X ST=X L=X O=X OU=X CN=X",
                "public_key_algo": "XXX",
                "public_key_bits": X,
            }

        Raises:
            ApiError: If status_code != 201.

        """
        request_files = {
            "file": certificate,
            "key_file": key
        }
        response = requests.post(
            "https://{}/api/net/certificate".format(self.ip),
            headers=self.headers, verify=False,
            proxies=self.PROXIES, files=request_files
        )
        if response.status_code == 201:
            logger.debug("Set certificate for Gcenter {} [OK]".format(self.ip))
            return response.json()
        else:
            raise ApiError(
                "Set certificate for Gcenter {} [FAILED]".
                format(self.ip), response.text, response.status_code, response.reason
            )

    @check_auth
    @minimum_version("2.5.3.101")
    def get_netdata(self) -> dict:
        """Get netdata settings.

        Returns:
            Return netdata settings with format::

            {
              "enabled": false,
              "dest_ip": null,
              "dest_port": 19999,
              "api_key": null,
              "tls": false,
              "cert_check": false,
              "parent_certificate": null,
              "iface": null,
              "certificate_details": {
                "version": null,
                "version_hexa": null,
                "serial_num": null,
                "sign_algo": null,
                "issuer": null,
                "not_before": null,
                "not_after": null,
                "subject": null,
                "public_key_algo": null,
                "public_key_bits": null
              }
            }

        Raises:
            ApiError: If status_code != 200.

        """
        response = self._get(endpoint="/api/data-export/netdata", headers=self.headers)
        if response[2] == 200:
            logger.debug("Get netdata settings on gcenter {} [OK]".format(self.ip))
            return json.loads(response[1])
        else:
            raise ApiError(
                "Get netdata settings on gcenter {} [FAILED]".
                format(self.ip), *response[1:]
            )

    @check_auth
    @minimum_version("2.5.3.101")
    def set_netdata(self,  ip: str = "", port: int = 80, key: str = "",
                    enabled: bool = True, tls: bool = False,
                    cert_check: bool = False, iface: str = "mgmt0",
                    cert: str = None) -> dict:
        """Set netdata settings.

        Args:
            ip: Netdata server IP address.
            port: Netdata server port number.
            key: Netdata server API key.
            enabled: Set to True to enable Netdata export and False instead.
            tls: Set to True to enable tls and False instead.
            cert_check: Set to True to enable server/client certificate check and False instead.
            iface: Set the Gcenter output interface.
            cert: Set the client certificate in PEM string format.

        Returns:
            Return netdata settings with format::

            {
              "enabled": false,
              "dest_ip": null,
              "dest_port": 19999,
              "api_key": null,
              "tls": false,
              "cert_check": false,
              "parent_certificate": null,
              "iface": null,
              "certificate_details": {
                "version": null,
                "version_hexa": null,
                "serial_num": null,
                "sign_algo": null,
                "issuer": null,
                "not_before": null,
                "not_after": null,
                "subject": null,
                "public_key_algo": null,
                "public_key_bits": null
              }
            }

        Raises:
            ApiError: If status_code != 200.

        """
        if enabled and ip == "" and key == "":
            raise ValueError("Must be set an IP address and an API key.")
        elif enabled:
            request_data = {
                "enabled": enabled,
                "dest_ip": ip,
                "dest_port": port,
                "api_key": key,
                "tls": tls,
                "cert_check": cert_check,
                "iface": iface
            }
        else:
            request_data = {
                "enabled": enabled,
                "tls": tls,
                "cert_check": cert_check
            }
        if cert is not None:
            request_data["parent_certificate"] = cert
        response = self._put(endpoint="/api/data-export/netdata",
                             data=request_data, headers=self.headers)
        if response[2] == 200:
            logger.debug("Set netdata settings on gcenter {} [OK]".format(self.ip))
            return json.loads(response[1])
        else:
            raise ApiError(
                "Set netdata settings on gcenter {} [FAILED]".
                format(self.ip), *response[1:]
            )

    @check_auth
    @minimum_version("2.5.3.101")
    def get_syslog(self, id: int) -> dict:
        """Get syslog settings.

        Args:
            id: Syslog configuration identifier. Can be 1 or 2.

        Returns:
            Return syslog settings with format::

            {
              "enabled": false,
              "name": "First logging server",
              "last_change": "2021-04-20 12:40:29.896724",
              "interface": "mgmt0",
              "scope": "alerts",
              "codec": "json",
              "protocol": "tcp",
              "facility": "kernel",
              "severity": "emergency",
              "rfc": "3164",
              "hostname": "localhost",
              "port": 514,
              "check_certificate": false,
              "enable_tls": false,
              "certificate": null,
              "certificate_key": null,
              "certificate_authority": null,
              "filters_ips": [],
              "filters_gcap": [],
              "filters_protocols": [],
              "certificate_details": {
                "version": null,
                "version_hexa": null,
                "serial_num": null,
                "sign_algo": null,
                "issuer": null,
                "not_before": null,
                "not_after": null,
                "subject": null,
                "public_key_algo": null,
                "public_key_bits": null
              },
              "certificate_authority_details": {
                "version": null,
                "version_hexa": null,
                "serial_num": null,
                "sign_algo": null,
                "issuer": null,
                "not_before": null,
                "not_after": null,
                "subject": null,
                "public_key_algo": null,
                "public_key_bits": null
              },
              "custom_fields": []
            }

        Raises:
            ApiError: If status_code != 200.

        """
        response = self._get(endpoint="/api/data-export/syslog/{}".format(id),
                             headers=self.headers)
        if response[2] == 200:
            logger.debug("Get syslog settings on gcenter {} [OK]".format(self.ip))
            return json.loads(response[1])
        else:
            raise ApiError(
                "Get syslog settings on gcenter {} [FAILED]".
                format(self.ip), *response[1:]
            )

    @check_auth
    @minimum_version("2.5.3.101")
    def disable_syslog(self, id: int) -> dict:
        """Disable syslog export.

        Args:
            id: Syslog configuration identifier. Can be 1 or 2.

        Returns:
            Return a dictionnary with the same format as the get_syslog
            function.

        Raises:
            ApiError: If status_code != 200.

        """
        request_data = self.get_syslog(id)
        request_data["enabled"] = False
        response = self._put(endpoint="/api/data-export/syslog/{}".format(id),
                             data=request_data, headers=self.headers)
        if response[2] == 200:
            logger.debug("Disable syslog export on gcenter {} [OK]".format(self.ip))
            return json.loads(response[1])
        else:
            raise ApiError(
                "Disable syslog export on gcenter {} [FAILED]".
                format(self.ip), *response[1:]
            )

    @check_auth
    @minimum_version("2.5.3.101")
    def set_syslog_general(self,
                           id: int,
                           hostname: str,
                           name: str = "Syslog API",
                           port: int = 514,
                           interface: str = "mgmt0",
                           scope: str = "alerts",
                           codec: str = "json",
                           protocol: str = "tcp",
                           facility: str = "syslogd",
                           severity: bool = "warning",
                           rfc: int = 3164) -> dict:
        """Set syslog general tab settings.

        Args:
            id: Syslog configuration identifier. Can be 1 or 2.
            hostname: IP address or fqdn of the syslog server.
            port: Port of the syslog server.
            interface: Gcenter's output interface.
            scope: Which alerts to send to the syslog server.
                    Values: [alerts,all]
            codec: Syslog codec.
                    Values: [idmef,json]
            protocol: Syslog transport protocol.
                        Values: [tcp,udp,ssl-tcp]
            facility: Syslog facility.
                        Values: [kernel,user-level,mail,daemon,syslogd,syslogd,
                                security/authorization,line printer,
                                line printer,network news,uucp]
            severity: Syslog severity.
                        Values: [emergency,alert,critical,error,warning,notice,
                                informational,debug]
            rfc: Syslog rfc.
                    Values: [3164,5424]

        Returns:
            Return a dictionnary with the same format as the get_syslog
            function.

        Raises:
            ApiError: If status_code != 200.
            TypeError: If protocol, scope, codec, rfc, severity, facility or id
                        value doesn't exist.

        """
        id_values = [1, 2]
        scope_values = ["alerts", "all"]
        codec_values = ["idmef", "json"]
        protocol_values = ["tcp", "udp", "ssl-tcp"]
        rfc_values = [3164, 5424]
        severity_values = ["emergency", "alert", "critical", "error",
                           "warning", "notice", "informational", "debug"]
        facility_values = ["kernel", "user-level", "mail", "daemon", "syslogd",
                           "syslogd", "security/authorization", "line printer",
                           "network news", "uucp"]
        gcenter_interfaces = [int["name"] for int in self.get_gcenter_interfaces()]
        if id not in id_values:
            raise TypeError("Syslog id must be between: {}".
                            format(id_values))
        if scope not in scope_values:
            raise TypeError("Syslog scope must be between: {}".
                            format(scope_values))
        if codec not in codec_values:
            raise TypeError("Syslog codec must be between: {}".
                            format(codec_values))
        if protocol not in protocol_values:
            raise TypeError("Syslog protocol must be between: {}".
                            format(protocol_values))
        if rfc not in rfc_values:
            raise TypeError("Syslog rfc must be between: {}".
                            format(rfc_values))
        if severity not in severity_values:
            raise TypeError("Syslog severity must be between: {}".
                            format(severity_values))
        if facility not in facility_values:
            raise TypeError("Syslog facility must be between: {}".
                            format(facility_values))
        if interface not in gcenter_interfaces:
            raise TypeError("Syslog interface must be between: {}".
                            format(gcenter_interfaces))
        request_data = self.get_syslog(id)
        if (request_data["enable_tls"] is True and request_data["protocol"]
                != "ssl-tls" and request_data["rfc"] != 5424):
            raise TypeError(
                "Rfc 5424 and protocol ss-tls must be used when TLS is enabled"
            )
        request_data["name"] = name
        request_data["hostname"] = hostname
        request_data["port"] = port
        request_data["interface"] = interface
        request_data["scope"] = scope
        request_data["codec"] = codec
        request_data["protocol"] = protocol
        request_data["facility"] = facility
        request_data["severity"] = severity
        request_data["rfc"] = rfc
        request_data["enabled"] = True
        response = self._put(endpoint="/api/data-export/syslog/{}".format(id),
                             data=request_data, headers=self.headers)
        if response[2] == 200:
            logger.debug("Set syslog general settings on gcenter {} [OK]".format(self.ip))
            return json.loads(response[1])
        else:
            raise ApiError(
                "Set syslog general settings on gcenter {} [FAILED]".
                format(self.ip), *response[1:]
            )

    @check_auth
    @minimum_version("2.5.3.101")
    def clear_syslog_filters(self, id: int) -> dict:
        """Clear all the syslog filters configuration.
        Set IP,GCAP,PROTOCOLS and CUSTUM FIELDS filters to Nothing.

        Args:
            id: Syslog configuration identifier. Can be 1 or 2.

        Returns:
            Return a dictionnary with the same format as the get_syslog
            function.

        Raises:
            ApiError: If status_code != 200.

        """
        request_data = self.get_syslog(id)
        if request_data["interface"] is None:
            request_data["interface"] = "mgmt0"
        request_data["filters_ips"] = []
        request_data["filters_gcap"] = []
        request_data["filters_protocols"] = []
        request_data["custom_fields"] = []
        response = self._put(endpoint="/api/data-export/syslog/{}".format(id),
                             data=request_data, headers=self.headers)
        if response[2] == 200:
            logger.debug("Clear all the syslog filters settings [OK]")
            return json.loads(response[1])
        else:
            raise ApiError(
                "Clear all the syslog filters settings [FAILED]", *response[1:]
            )

    @check_auth
    @minimum_version("2.5.3.101")
    def set_syslog_filters(self,
                           id: int,
                           ips: list = [],
                           gcap: list = [],
                           protocols: list = [],
                           fields: list = []) -> dict:
        """Set syslog filters tab settings.

        Args:
            id (int): Syslog configuration identifier. Can be 1 or 2.
            ips (list): List of IP address with format: X.X.X.X or X.X.X.X/Y.
            gcap (list): List of gcap with fqdn format: gcap.gatewatcher.com.
            protocols (list): List of protocols. Values: [dhcp, dns, ftp, http,
                    ikev2, krb5, nfs, smb, smtp, ssh, tftp, tls, ntp]
            fields (list): List of custum fields dictionnary. Format:
                    [{"name": "string", "value": "string"}]

        Returns:
            Return a dictionnary with the same format as the get_syslog
            function.

        Raises:
            ApiError: If status_code != 200.
            TypeError: If protocols value doesn't exist.

        """
        protocol_values = ["dhcp", "dns", "ftp", "http", "ikev2", "krb5",
                           "nfs", "smb", "smtp", "ssh", "tftp", "tls", "ntp"]
        for proto in protocols:
            if proto not in protocol_values:
                raise TypeError("Syslog protocol must be between: {}".
                                format(protocol_values))
        request_data = self.get_syslog(id)
        if request_data["interface"] is None:
            request_data["interface"] = "mgmt0"
        request_data["filters_ips"] = list(set(
            [*request_data["filters_ips"], *ips]
        ))
        request_data["filters_gcap"] = list(set(
            [*request_data["filters_gcap"], *gcap]
        ))
        request_data["filters_protocols"] = list(set(
            [*request_data["filters_protocols"], *protocols]
        ))
        request_data["custom_fields"] = list(set(
            [*request_data["custom_fields"], *fields]
        ))
        response = self._put(endpoint="/api/data-export/syslog/{}".format(id),
                             data=request_data, headers=self.headers)
        if response[2] == 200:
            logger.debug("Set syslog filters settings [OK]")
            return json.loads(response[1])
        else:
            raise ApiError(
                "Set syslog filters settings [FAILED]", *response[1:]
            )

    @check_auth
    @minimum_version("2.5.3.101")
    def set_syslog_certificate(self,
                               id: int,
                               cert: str = "",
                               cert_key: str = "",
                               ca: str = "",
                               cert_check: bool = False,
                               tls: bool = True) -> dict:
        """Set syslog certificate tab settings.

        Args:
            id: Syslog configuration identifier. Can be 1 or 2.
            cert: Syslog client certificate in PEM format.
            cert_key: Syslog client certificate private key in PEM format.
            ca: Certificate of the Certification Authority in PEM format.
            cert_check: Set to True to enabled the server certificate
                        verification.
            tls: Set to True to enabled tls and False instead.

        Returns:
            Return a dictionnary with the same format as the get_syslog
            function.

        Raises:
            ApiError: If status_code != 200.

        """
        request_data = self.get_syslog(id)
        if request_data["interface"] is None:
            request_data["interface"] = "mgmt0"
        request_data["certificate"] = cert
        request_data["certificate_key"] = cert_key
        request_data["certificate_authority"] = ca
        request_data["check_certificate"] = cert_check
        request_data["enable_tls"] = tls
        if tls:
            request_data["rfc"] = 5424
            request_data["protocol"] = "ssl-tcp"
        response = self._put(endpoint="/api/data-export/syslog/{}".format(id),
                             data=request_data, headers=self.headers)
        if response[2] == 200:
            logger.debug("Set syslog certificate settings [OK]")
            return json.loads(response[1])
        else:
            raise ApiError(
                "Set syslog certificate settings [FAILED]", *response[1:]
            )

    @check_auth
    @minimum_version("2.5.3.101")
    def get_es_query(self, index: str, query: dict) -> list:
        """Get results of an elasticsearch query.

        Args:
            index: Index name between suricata, codebreaker, malware,
                    netdata, syslog.
            query: Query in dictionnary format.

        Returns:
            Return a list of entry in dictionnary format: [{},{},...].

        Raises:
            ApiError: If status_code != 200.
            TypeError: If index value doesn't exist.

        """
        index_values = ["suricata", "codebreaker", "malware", "netdata",
                        "syslog"]
        if index not in index_values:
            raise TypeError("Index value must be between: {}".
                            format(index_values))
        response = self._post(
            endpoint="/api/data/es/search?index={}".format(index),
            data=query, headers=self.headers
        )
        if response[2] == 200:
            logger.debug("Get elasticsearch results for index {} [OK]".format(index))
            return json.loads(response[1])
        else:
            raise ApiError(
                "Get elasticsearch results for index {} [FAILED]".
                format(index), *response[1:]
            )

    @check_auth
    @minimum_version("2.5.3.101")
    def get_es_count(self, index: str, query: dict) -> int:
        """Get the number of result of an elasticsearch query.

        Args:
            index: Index name between suricata, codebreaker, malware,
                    netdata, syslog.
            query: Query in dictionnary format.

        Returns:
            Return the number of result found.

        Raises:
            ApiError: If status_code != 200.
            TypeError: If index value doesn't exist.

        """
        index_values = ["suricata", "codebreaker", "malware", "netdata",
                        "syslog"]
        if index not in index_values:
            raise TypeError("Index value must be between: {}".
                            format(index_values))
        response = self._post(
            endpoint="/api/data/es/search?index={}".format(index),
            data=query, headers=self.headers
        )
        if response[2] == 200:
            logger.debug("Get elasticsearch results for index {} [OK]".
                         format(index))
            return json.loads(response[1])["hits"]["total"]["value"]
        else:
            raise ApiError(
                "Get elasticsearch results for index {} [FAILED]".
                format(index), *response[1:]
            )
