"""GWAPI decorators."""
from . import errors, utils
import functools
from datetime import datetime


def check_auth(func: callable) -> callable:
    """Check if the user is already authenticated to the Gcenter.

    Args:
        func: Function to be checked.

    Returns:
        Return a new function that check of the user is authenticated.

    """
    @functools.wraps(func)
    def wrapper(self, *args, **kwargs) -> callable:
        if self.token is None:
            raise errors.AuthenticationError(
                "You must be logged in to the Gcenter to exetute this action !"
            )
        elif datetime.utcnow() > datetime.fromisoformat(self.token_expiration_date):
            raise errors.AuthenticationError("Your token has expired !")
        return func(self, *args, **kwargs)
    return wrapper


def minimum_version(version: str) -> callable:
    """Check if the function is compatible with the Gcenter version.

    Args:
        version: The minimum version that is compatible with the function.

    Returns:
        Return a new function that check the Gcenter version.

    """
    def decorator(func) -> callable:
        @functools.wraps(func)
        def wrapper(self, *args, **kwargs) -> callable:
            if utils.version_lt(self.version, version):
                raise errors.InvalidVersion(
                    '{} is not available for version < {}'.format(
                        func.__name__, version
                    )
                )
            return func(self, *args, **kwargs)
        return wrapper
    return decorator
